#ifndef POKEPLATINUM_UNK_02009714_H
#define POKEPLATINUM_UNK_02009714_H

#include "struct_decls/struct_02006C24_decl.h"
#include "struct_decls/struct_02009714_decl.h"
#include "struct_defs/struct_02009CFC.h"
#include "struct_decls/struct_02009DC8_decl.h"
#include "struct_decls/struct_02009F38_decl.h"

#include <nnsys.h>

UnkStruct_02009714 * sub_02009714(int param0, int param1, int param2);
void sub_02009754(UnkStruct_02009714 * param0);
UnkStruct_02009DC8 * sub_02009794(UnkStruct_02009714 * param0, const UnkStruct_02009F38 * param1, int param2, int param3);
UnkStruct_02009DC8 * sub_0200985C(UnkStruct_02009714 * param0, int param1, int param2, BOOL param3, int param4, int param5, int param6);
UnkStruct_02009DC8 * sub_020098B8(UnkStruct_02009714 * param0, int param1, int param2, BOOL param3, int param4, int param5, int param6, int param7);
UnkStruct_02009DC8 * sub_02009918(UnkStruct_02009714 * param0, int param1, int param2, BOOL param3, int param4, int param5, int param6);
void sub_02009968(UnkStruct_02009714 * param0, UnkStruct_02009DC8 * param1, int param2, int param3, BOOL param4, int param5);
void sub_020099D4(UnkStruct_02009714 * param0, UnkStruct_02009DC8 * param1, int param2, int param3, BOOL param4, int param5);
UnkStruct_02009DC8 * sub_02009A4C(UnkStruct_02009714 * param0, NARC * param1, int param2, BOOL param3, int param4, int param5, int param6);
UnkStruct_02009DC8 * sub_02009AA8(UnkStruct_02009714 * param0, NARC * param1, int param2, BOOL param3, int param4, int param5, int param6, int param7);
UnkStruct_02009DC8 * sub_02009B04(UnkStruct_02009714 * param0, NARC * param1, int param2, BOOL param3, int param4, int param5, int param6, int param7);
UnkStruct_02009DC8 * sub_02009B64(UnkStruct_02009714 * param0, NARC * param1, int param2, BOOL param3, int param4, int param5, int param6, int param7, int param8);
UnkStruct_02009DC8 * sub_02009BC4(UnkStruct_02009714 * param0, NARC * param1, int param2, BOOL param3, int param4, int param5, int param6);
void sub_02009C14(UnkStruct_02009714 * param0, UnkStruct_02009DC8 * param1, NARC * param2, int param3, BOOL param4, int param5);
int sub_02009C80(UnkStruct_02009714 * param0, const UnkStruct_02009F38 * param1, UnkStruct_02009CFC * param2, int param3);
void sub_02009CB4(UnkStruct_02009714 * param0, const UnkStruct_02009F38 * param1, int param2, int param3, UnkStruct_02009CFC * param4, int param5);
UnkStruct_02009CFC * sub_02009CFC(int param0, int param1);
void sub_02009D20(UnkStruct_02009CFC * param0);
BOOL sub_02009D34(const UnkStruct_02009714 * param0, int param1);
void sub_02009D4C(UnkStruct_02009DC8 * param0);
void sub_02009D68(UnkStruct_02009714 * param0, UnkStruct_02009DC8 * param1);
void sub_02009D9C(UnkStruct_02009714 * param0);
UnkStruct_02009DC8 * sub_02009DC8(const UnkStruct_02009714 * param0, int param1);
int sub_02009E08(const UnkStruct_02009DC8 * param0);
NNSG2dCharacterData * sub_02009E1C(const UnkStruct_02009DC8 * param0);
NNSG2dPaletteData * sub_02009E34(const UnkStruct_02009DC8 * param0);
NNSG2dCellDataBank * sub_02009E4C(const UnkStruct_02009DC8 * param0);
NNSG2dCellAnimBankData * sub_02009E64(const UnkStruct_02009DC8 * param0);
NNSG2dMultiCellDataBank * sub_02009E7C(const UnkStruct_02009DC8 * param0);
NNSG2dMultiCellAnimBankData * sub_02009E94(const UnkStruct_02009DC8 * param0);
int sub_02009EAC(const UnkStruct_02009DC8 * param0);
int sub_02009EBC(const UnkStruct_02009DC8 * param0);
int sub_02009EE8(const UnkStruct_02009DC8 * param0);
void sub_02009F08(UnkStruct_02009DC8 * param0, int param1);
int sub_02009F34(void);
UnkStruct_02009F38 * sub_02009F38(UnkStruct_02009F38 * param0, int param1);
void sub_02009F40(const void * param0, UnkStruct_02009F38 * param1, int param2);
void sub_02009F8C(UnkStruct_02009F38 * param0);
int sub_02009FA4(const UnkStruct_02009F38 * param0);
int sub_02009FB4(const UnkStruct_02009F38 * param0, int param1);
int sub_02009FE8(const UnkStruct_02009F38 * param0, int param1);
int sub_0200A014(const UnkStruct_02009F38 * param0, int param1);
int sub_0200A040(const UnkStruct_02009F38 * param0, int param1);
int sub_0200A074(const UnkStruct_02009F38 * param0, int param1);

#endif // POKEPLATINUM_UNK_02009714_H
